#!/usr/bin/perl -w
#
# This is e-mail autoreplyer
# 
# by 2006 rtrebula@prefis.sk
#
# This code is public domain. No copyright, no warranty.
#

use DBI;
use Email::Simple;
use Email::Simple::Creator;
use Email::Send;
use Email::Address;
use strict;
$Email::Send::Sendmail::SENDMAIL = '/usr/sbin/sendmail';
$::domain="domain.tld";

my $user=$ENV{"LOGNAME"};
die "Cannot find out current username" unless $user;

$::dbh = DBI->connect('DBI:mysql:vacation', 'user', 'password')
	or die "Couldn't connect to database: " . DBI->errstr;

my $mail_text=&read_from_stdin();

my $reply_message=&check_vacation_active($user);
if($reply_message) {
	my $mail = Email::Simple->new($mail_text);

	if(&is_bulk($mail)) {
		print STDERR "SMUOOOR: Is bulk message, not replying.\n";
		&clean_up_exit;
	}
	if(not &is_our_alias($mail, $user)) {
		print STDERR "SMUOOOR: Not sent to us, not replying.\n";
		&clean_up_exit;
	}
	if(&known_sender($mail, $user)) {
		print STDERR "SMUOOOR: Sender is alredy notified, not replying.\n";
		&clean_up_exit;
	}
	print STDERR "SMUOOOR: User $user out of office, responding\n";
	my $reply = &create_reply($mail, $user, $reply_message);
	
	my $sender = Email::Send->new({mailer => 'Sendmail'});
	$sender->send($reply);

	# print STDERR $reply->as_string;

} else {
	print STDERR "SMUOOOR: User $user not out of office - not responding\n";
}

&clean_up_exit;

### 
### End of main
###

sub read_from_stdin {
	my @mail;
	while(<>) {push @mail, $_;}
	return join("", @mail);
}

sub check_vacation_active {
	my $user=shift;
	my $respond_message=0;

	my $sql=<<EOF;
select vacation_active, vacation_message from User
	where login="$user"
EOF

	my $sth = $::dbh->prepare($sql)
		or die "Couldn't prepare SQL statement: " . DBI->errstr;
	$sth->execute()
		or die "Couldn't execute query: " . DBI->errstr;

	my $vacation; my $message;
	while (($vacation, $message) = $sth->fetchrow_array()) {
		if($vacation) {
			$respond_message="$message";
		}
	}
	return $respond_message;
}

sub is_bulk {
	my $mail=shift;
	if ($mail->header("X-Mail-Vacation")) { return 1;}
	if ($mail->header("X-Mailing-List")) { return 1;}
	if ($mail->header("X-Spam-Status") =~ /Yes/) { return 1;}
	my $precedence = $mail->header("Precedence");
	if($precedence =~ /junk/ or
		$precedence =~ /bulk/ or
		$precedence =~ /list/) { return 1;}

	if(&bulk_sender($mail->header("From"))) { return 1;}
	if(&bulk_sender($mail->header("Reply-To"))) { return 1;}
	if(&bulk_sender($mail->header("Return-Path"))) {return 1;}
	
	return 0;
}

sub bulk_sender {
	foreach my $from (@_) {
		return 1
			if ($from =~ /-request/ or
				$from =~ /postmaster/ or
				$from =~ /uucp/ or
				$from =~ /mailer/ or 
				$from =~ /-relay/);
	}
	return 0;
}

sub is_our_alias {
	my ($mail, $user) = @_;
	
	# TODO check in the Alias table
	my $to_header=$mail->header("To");
	my @addresses=Email::Address->parse($to_header);
	foreach my $to (@addresses) {
		return 1
			if($to->address =~ qr/$user/);
	}
	return 0;
}

sub known_sender {
	my ($mail, $user) = @_;
	my $known;

	my $from_header=$mail->header("From");
	my @addresses=Email::Address->parse($from_header);
	return 1
		unless(defined($addresses[0]));
	my $from=$addresses[0]->address;
	
	my $sql=<<EOF;
select date from KnownSender
	where user="$user" and sender="$from"
EOF

	my $sth = $::dbh->prepare($sql)
		or die "Couldn't prepare SQL statement: " . DBI->errstr;
	$sth->execute()
		or die "Couldn't execute query: " . DBI->errstr;

	my $date_seen;
	while (($date_seen) = $sth->fetchrow_array()) {
		$known=1;
	}
	if($known) {
		return 1;
	} else {
		my $sql=<<EOF;
insert into KnownSender (user, sender, date)
	values ("$user", "$from", NOW())
EOF

		$sth = $::dbh->prepare($sql)
			or die "Couldn't prepare SQL statement: " . DBI->errstr;
		$sth->execute()
			or die "Couldn't execute query: " . DBI->errstr;
	
		return 0;
	}
}

sub create_reply {
	my ($src,$user, $body)=@_;
	my $to=$src->header("Reply-To");
	$to=$src->header("From") unless $to;
	my $from = $user . "@" . $::domain;
	
	my $reply=Email::Simple->create(
		header=> [
			"To" 	=> $to,
			"From"	=> $from,
			"Subject"=> "Out Of Office Reply",
			"X-Mail-Vacation" => "smuooor",
			"Precedence" => "junk"],
		body=>$body
	);

	return $reply;
}

sub clean_up_exit {
	if(defined($::dbh)) {
		$::dbh->disconnect;
	}
	exit;
}

